c
c     Sample program to solve a block tridiagonal matrix
c     using the LU factorization. 
c
c     Also compute the residual in the block form Ax-b
c
c     double precision version 
c
c     Veronese@math.utah.edu march 97
c
c   
c     Note: Derived from LAPACK and blas routines. 
c           Makes use of a block tridiagonal solver. 
c           The diagonal blocks (A) must not be singular.
c           the sub and super block MAY be singular. 
c          
c     Note2: change format statments if you change m or n
c 
c
c------------------------------------------------------------

      integer m,n,k,i,j,ier
      parameter(m=2)    ! order of each block
      parameter(n=5)    ! number of blocks 
      double precision   a(m,m,n),b(m,m,n),c(m,m,n)
      double precision   a2(m,m,n),b2(m,m,n),c2(m,m,n)
      double precision   bloa(m,m),blob(m,m),bloc(m,m)
      double precision   ra(m),rb(m),rc(m),r(m,n)
      double precision  ya(m),yb(m),yc(m)
      double precision  f(m*n),bf(m,n)
      double precision  y(m*n),by(m,n)
     
     
      integer ip(m,m,n)


    
c   fill in a containing diagonal blocks.

       do k=1,n
       do i=1,m
       do j=1,m
         a(i,j,k)=0.d0 
         b(i,j,k)=0.d0
         c(i,j,k)=0.d0

c          a(i,j,k)=1+0.1
        if(i.eq.j)            a(i,j,k)=1+0.1
c        if(i.eq.j+1)          a(i,j,k)=1
c        if(i.eq.j-1)          a(i,j,k)=1
       enddo 
       enddo 
       enddo 


c   fill in b  containing super diagonal blocks.
       do k=1,n
       do i=1,m
       do j=1,m
       if(i.eq.j)   b(i,j,k)=0.1/2.d0
c          b(m,1,k)=1.d0  
c          b(1,m,k)=1.d0 
           enddo 
       enddo 
       enddo 
 
c   fill in c  containing sub diagonal blocks.
       do k=1,n
       do i=1,m
       do j=1,m
       if(i.eq.j)   c(i,j,k)=0.1/2.0
c          c(1,m,k)=1.d0  
c          c(m,1,k)=1.d0  
       enddo 
       enddo 
       enddo
c
c    make sure c(:,:,1) and b(:,:,n) are zero
c
      do i=1,m
       do j=1,m
          c(i,j,1)=0.d0
          b(i,j,n)=0.d0  
       enddo 
       enddo 
c
c    duplicate a,b,c 
c
       do k=1,n
       do i=1,m
       do j=1,m
          a2(i,j,k)=a(i,j,k)
          b2(i,j,k)=b(i,j,k)
          c2(i,j,k)=c(i,j,k)
       enddo 
       enddo 
       enddo 

       print*, 'before decomposition'
       call wout(a,b,c,n,m)
c
c
c     decompose LU 

       call  decbt (m, n, a, b, c, ip, ier)

       print*, 'after  decomposition'
       call wout(a,b,c,n,m)

       print*, 'IER=',ier

c     build right hand side

        do k=1,n
        do i=1,m
          by(i,k)=i*k
          bf(i,k)=by(i,k)
        enddo 
        enddo 

c      solve system 
 

       call dsolbt(m, n, a, b, c,by, ip)

c    

      do k=1,n
       do i=1,m
             print*, 'sol',i,k,i+(k-1)*m,by(i,k)
       enddo
      enddo


c     check errors i.e. do mat*x-rsh
c     (by performing multiplication by blocks)


       do k=1,n

       do i=1,m
       do j=1,m
         bloa(i,j)=a2(i,j,k)
         blob(i,j)=b2(i,j,k)
         bloc(i,j)=c2(i,j,k)
        enddo
       yc(i)=by(i,k-1)
       ya(i)=by(i,k)
       yb(i)=by(i,k+1)
       enddo
 
       call mul(bloa,ya,ra,m)
       call mul(blob,yb,rb,m)
       call mul(bloc,yc,rc,m)
       
      do i=1,m
      r(i,k)=ra(i)+rb(i)+rc(i)
      r(i,k)=bf(i,k)-r(i,k)
      print*, 'residual',i,k,r(i,k)
      enddo
   
      enddo 

 
c       call wout(a,b,c,n,m)
       end 
c---------------------------------------------------------------
       subroutine wout(a,b,c,n,m)
c
c      write a block matrix to standard output
c

       integer m,n,k,i,j
      double precision  a(m,m,n),b(m,m,n),c(m,m,n)
     
c       k=1
c       do i=1,m
c          write(*,29)(a(i,j,k),j=1,m),(b(i,j,k),j=1,m)
c       enddo

       do k=1,n
       write(*,*) 'Block',k
       do i=1,m
          write(*,19)(c(i,j,k),j=1,m),
     .       (a(i,j,k),j=1,m),(b(i,j,k),j=1,m)
       enddo
       enddo
 
c c      k=n
c       do i=1,m
c          write(*,29)(c(i,j,k),j=1,m),(a(i,j,k),j=1,m)
c       enddo
     
   19   format ( 6(f13.5,1x)) 
   29   format ( 4(f13.5,1x))


        return 
        end
c---------------------------------------------------------------
        subroutine  mul(mat,x,y,m)
c
c         matrix * vector  y=mat*x
c
        integer m,n,k,i,j
       double precision  mat(m,m),x(m),y(m),sum

        do i=1,m
        sum = 0 
        do j=1,m
          sum=sum+mat(i,j)*x(j)
        enddo 
          y(i)=sum
        enddo 
        return 
        end
