c**********************************************************************
c
       subroutine functz(meq,mveq,p,of)
c ----------------------------------------------------------------------      
c
c     Note that meq and mveq are not used
c
c
c This subroutine computes the function values. The function is 
c
c   -(u_n+1-u_n)/ht +1/2d_x(u*+u_n)+1/2d_y(u**+u_n)+1/2d_z(u_n+u_n+1)
c
c This is equivalent to compute 
c
c     -(I-(ht/2)A3)u_n+1 + u** - (ht/2)A3u_n + (ht/2)(f(u_n+1)-f(u_n))
c
c Input: p = (p1,p2,p3,icol,irow) with dimension 3*meqnz+2. 
c        where    
c        p1 = u(icol,irow,(z1,...,znvar)1,...,(z1,...,znvar)nz)
c        p2 = u**(icol,irow,(z1,...,znvar)1,...,(z1,...,znvar)nz)
c        p3 = oldu(icol,irow,(z1,...,znvar)1,...,(z1,...,znvar)nz)
c Output: of with dimension meqnz in which the function values are stored in 
c ----------------------------------------------------------------------      
c
      include './params.inc'

      integer*4 l,k,icol,irow,iplane,meq,mveq
      integer*4 ind

      double precision p(3*meqnz+2)
      double precision of(meqnz)
      double precision u(nvar)
      double precision u0(nvar)
      double precision lambdz
      double precision fval1(nvar)                 !f(u^n)
      double precision fval2(nvar)                 !f(u^n+1)
      double precision t,ht,htx,hty,htz
      double precision diffz
      double precision g1,g2,g3,g4,g5,g6
c     double precision x,y,z

      character*10  bc
c
      external g1,g2,g3,g4,g5,g6
c
      common/ttime/t,ht,htx,hty,htz
      common/bound_cond/ bc
c
c---------------------------------------------------------------------
c     Compute compute -(u-u**) + ht*(f(u)-f(oldu))/2
c-------------------------------------------------------------------
      icol= idint(p(3*meqnz+1))
      irow= idint(p(3*meqnz+2))

      do k = 1,nz

         iplane=k
c
c Extract the jth component at current time
c
         do l = 1,nvar
            u(l) = p(l +(k-1)*nvar)
            u0(l) = p(l +(k-1)*nvar+2*meqnz)
         end do
c
c  compute -(u-u**) + ht*(f(u)-f(oldu))/2
c
         call func(u0,irow,icol,iplane,t-ht,fval1)
         call func(u,irow,icol,iplane,t,fval2)

         do l = 1,nvar
            of(l+(k-1)*nvar)=-(p(l+(k-1)*nvar)             !u
     .                      -p(l+(k-1)*nvar+meqnz))        !u**
     .                 +0.5d0*ht*(fval2(l)-fval1(l))
         end do
c
      end do
c
c ----------------------------------------------------------------
c  Add diffusion ht*A3(u - oldu)/2
c
c That is, add the contribution from the 'z-direction'
c ----------------------------------------------------------------
c
       do l = nstat+1, nvar           ! diffusing species
         lambdz = htz*par(3+3*(l-1))

         do  k = 2,nz-1
c
c         Compute ht*A3(u - oldu)/2 at inside points
c
            ind=l+(k-1)*nvar
            diffz =p(ind-nvar)+p(ind+nvar)-2.d0*p(ind)       ! d_z(u_n+1)
c
            diffz = diffz-(p(ind-nvar+2*meqnz )
     .            + p(ind+nvar+2*meqnz)-2.d0*p(ind+2*meqnz)) ! -d_z(u_n)
            diffz = 0.5d0*lambdz*diffz
c
            of(ind) = of(ind) + diffz
c
         enddo
  
c ----------------------------------------------------------------
c   Boundary conditions at the endpoints (z=0,1)
c-----------------------------------------------------------------    
c	'Bottom' boundary    
c
c        k=1

         ind=l
c
c  for Neumann BC
c
         if(bc .eq. 'neumann') then
            diffz =2.d0*(p(ind+nvar)-p(ind)
     .         - hz*g5((icol-1)*hx,(irow-1)*hy,t+ht))      ! d_z(u_n+1)
     .         - 2.d0*(p(ind+nvar+2*meqnz)-p(ind+2*meqnz)
     .         - hz*g5((icol-1)*hx,(irow-1)*hy,t))         ! d_z(u_n)
c
c for Dirichlet BC
c
         elseif(bc .eq. 'dirichlet') then
            diffz =(p(ind+nvar)-2.d0*p(ind)
     .         + g5((icol-1)*hx,(irow-1)*hy,t+ht))         ! d_z(u_n+1)
     .         - (p(ind+nvar+2*meqnz)-2.d0*p(ind+2*meqnz)
     .         + g5((icol-1)*hx,(irow-1)*hy,t))            ! d_z(u_n)
         endif
c
         diffz=0.5d0*lambdz*diffz
c     
         of(ind) = of(ind) + diffz
c
c-----------------------------------------------------------------    
c
c	'Top' boundary
c
c      k = nz                  

         ind=l+(nz-1)*nvar
c
c  for Neumann BC
c
         if(bc .eq. 'neumann') then
            diffz = 2.d0*(p(ind-nvar)-p(ind)
     .         + hz*g6((icol-1)*hx,(irow-1)*hy,t+ht))       ! d_z(u_n+1)
     .           - 2.d0*(p(ind-nvar+2*meqnz)-p(ind+2*meqnz)
     .         + hz*g6((icol-1)*hx,(irow-1)*hy,t))          ! d_z(u_n)
c
c for Dirichlet BC
c
         elseif(bc .eq. 'dirichlet') then
            diffz = (p(ind-nvar)-2.d0*p(ind)
     .         + g6((icol-1)*hx,(irow-1)*hy,t+ht))          ! d_z(u_n+1)
     .           - (p(ind-nvar+2*meqnz)-2.d0*p(ind+2*meqnz)
     .         + g6((icol-1)*hx,(irow-1)*hy,t))             ! d_z(u_n)
         endif
c
         diffz = 0.5d0*lambdz*diffz
c
         of(ind)  = of(ind) + diffz
c
      enddo
c
      return
      end
c
c********************************************************************
c  
      subroutine funcjac(u,irow,icol,iplane,t,fp)
c
c ----------------------------------------------------------------------      
c This subroutine is to compute the jacobian matrix of f(u) at u^n+1
c
c Input: u, variable values at time step n+1 wit dimension nvar
c        (icol,irow,iplane) grid pint in (x,y,z) space 
c        t, time
c Output: fp, nvar by nvar jacobian matrix of f(u) at u^n+1
c ----------------------------------------------------------------------      
c
      include './params.inc'

      integer*4 i,j,l,icol,irow,iplane
c     integer*4 ind

      double precision fp(nvar,nvar)            !jacobian matrix
      double precision u(nvar), t
      double precision fval1(nvar)
      double precision fval2(nvar)
      double precision par1(50)

      double precision eps, up(nvar)

      common/param/par1
c
c     Jacobian calculation
c
        eps = 1.d-6

        do i = 1, nvar
           do j = 1, nvar
c
c Compute f(up(l)) = f(ul+eps) and f(u(l)) = f(ul)

              do l = 1, nvar
                 if (l .eq. j) then
                    up(l)= u(l) + eps 
                 else
                    up(l)= u(l) 
                 endif
              enddo

              call func(u,irow,icol,iplane,t,fval1)    !f(ul)
              call func(up,irow,icol,iplane,t,fval2)   !f(ul+eps)
c
              fp(i,j) = (fval2(i) -fval1(i))/eps       !jacobian df_i/d(uj)
c
           enddo
        enddo
c
        return
        end
c
c ***************************************************************************
c  
      subroutine jac(n,u,savf,v,z)
      integer n
      double precision u(1),savf(n),v(n),z(n)
      write(6,*) 'Calling jac'
      return
      end
c
c***************************************************************************
c  
         subroutine psol (n, nu, u, savf, f, jac, wk, wmp, iwmp, x, ier)

      integer n,nu,ier
      double precision u(nu),savf(n),wk(1),iwmp(1),x, wmp(1)
      external jac,f 
      write(6,*) 'Calling psol'
      return
      end
c *************************************************************************
c
      subroutine pset (n, nu, u, savf, su, sf, wk, f, wmp, iwmp, ier)

      integer n,nu,ier
      double precision u(nu),savf(n),su(n),sf(n),wmp(1),iwmp(1),wk(1)

      external f
      write(6,*) 'Calling pset'
      return
      end
c
c **************************************************************************

