c******************************************************************
c
      subroutine setup(iu,ius,idr,nointflg,restart,
     .           tscale,filename1,
     .           filename2,filename3)
c
c --------------------------------------------------------------------
      include 'params.inc'
      include 'nksol.inc'
c
      logical nointflg
      logical restart
c
      integer i
      integer iu
      integer ius
      integer idr
      integer isolver     !solver for Z, 1.nksol, 2. block_trid, 3. tridiag
c
      double precision tscale   ! min^{-1} time scale
c
c     double precision htx,hty,htz
c     double precision ht,par1
      double precision par1
c
      character*15 filename1
      character*15 filename2
      character*15 filename3
      character*10  bc
      character*10 solverx,solvery,solverz
d     character*20 dt 
c
      real*8 tfinal
      integer nsteps
      common /limits/ tfinal, nsteps      
c
      common/bound_cond/ bc
      common /solver/ solverx,solvery,solverz
      common /param/ par1(50)
c
c   solver 
c
      solverx = 'tridiag'
      solvery = 'tridiag'
      solverz = 'nksol'                !default solver for Z
c
c   solver for Z
c
      print*, 'Solver for Z? (1. nksol, 2. block_trid, 3. tridiag) '
      read(*,*) isolver
      if(isolver .eq. 1) then
         solverz = 'nksol'
      elseif(isolver .eq. 2) then
         solverz = 'block_trid'
      elseif(isolver .eq. 3) then
         solverz = 'tridiag'
      endif
      write(*,*) 'Sol method for X - Y -Z is ', solverx,solvery,solverz
      print *,'Enter number of time steps:'
      read (*,*) nsteps
      if (nsteps .le. 0) stop 'nsteps must be positive'
      tfinal = dfloat(nsteps)*ht0

c
c     setup boundary condition
c
      if(ibc .eq. 0) then
         bc = 'neumann'
      elseif(ibc .eq. 2) then
         bc = 'dirichlet'
      endif
c
c     Initialize the par(.) array
c
      do i = 1,3*nvar
         par(i) = 0.d0
      enddo
c     
c Read the diffusion coefficients into par(.) from the data file 
c (data.dif)
c
      open (unit = 21, file = 'data.dif', 
     . status = 'unknown')

      do i = 1+nstat, nvar
      read (21, *) par(1+(i-1)*3), par(2+(i-1)*3),par(3+(i-1)*3)
      enddo
      close (unit = 21)

c     
c Read the kinetic parameters into par1(.) from the data file (data.kin)
c
      open (unit = 20, file = 'data.kin', 
     . status = 'unknown')

      do i = 1,nparam
      read (20, *) par1(i)
      enddo
      close (unit = 20)

c-----------------------------------------------------------------------
c
      iu = 14
      ius = 18
      idr = 19

c      filename1 = 'fort.stat'
c      filename2 = 'fort.soln'
c      filename3 = 'fort.drop'

      filename1 = 'fort.14'
      filename2 = 'temp.soln'
      filename3 = 'soln2d'


      if(iu.ne.6) open(unit=iu,file=filename1,status='unknown')
      open(unit=ius,file=filename2,status='unknown')

c
c The next flag will allow the user to send the final output of the 
c parabolic solver as an initial guess for an elliptic solver.
c
      nointflg = .false.     ! skip time integration => true
      restart = .false.      ! start from drop file (true)
c
      call paramout (iu,tfinal,nointflg,restart,
     .               filename1,filename2,filename3)
c
      return
      end

c**********************************************************************
c
      Subroutine paramout(iu,nointflg,restart,
     .               filename1,filename2,filename3)
c
c----------------------------------------------------------------------
c
      include 'params.inc'
      include 'nksol.inc'
c
      integer iu, i
      integer total
      integer totalb
c
      double precision par1, tfinal
c
      logical nointflg
      logical restart
c
      character*10  bc
      character*10 solverx,solvery,solverz
      character*20 dt 
      character*15 filename1
      character*15 filename2
      character*15 filename3
c
      common/bound_cond/ bc
      common /solver/ solverx,solvery,solverz
      common /param/ par1(50)
c
c-----------------------------------------------------------------------
c Record the parameters in the file fort.14, (iu = 14 )
c     
      dt = '-no-date-available-'
c     call fdate(dt)
c     
      write(iu,46000) dt(1:11), dt(13:20)
c    
      write(iu,*) 'parameters for kinetics and initial data'
      write(iu,30000) (par1(i),i = 1,nparam)
      write(iu,*) 
c    
      write(iu,*) 'diffusion coefficients'
      write(iu,30001) (par(i),i = 1+3*nstat,3*nvar)
      write(iu,*) 
c     
      write(iu,100) sizex,sizey,sizez,nx,ny,nz,ht0,jn,tfinal,
     . tstart
c     
c
 100  format(1x,//,' Discretization information ',/,1x,
     $     ' Length of domain  in x,y,z        ',3f8.4,/,1x,
     $     ' Number of subdivisions (x,y,z)    ',3i4,/,1x,
     $     ' Time step                         ',1pd11.5,/,1x,
     $     ' Frequency of time output          ',i3,/,1x,
     $     ' Final time                        ',1pd11.5,/,1x,
     $     ' Time of first output point        ',1pd11.5,/,1x,//,
     .     1x)
c
30000 format(1x,5(1pe12.3))
30001 format(1x,3(1pe12.3))
46000 format(8x,'Written on ',a11,' at ',a8,'  from adi3d ',/)
c
c The arrays for this code can get large.  The following
c prints out the array size data into the stats file
c
      write(iu,*)
     . 'Array sizes (double precision only): len(u) = length of u'
      write(iu,*)'len(v) = ',neqn
      write(iu,*)'len(rwork) = ',lrw
      write(iu,*)'len(iwork) = ',liw
      write(iu,*)'len(rhsf) = ',neqn
      write(iu,*)'len(sf) = ',neqn
      write(iu,*)'len(su) = ',neqn
      write(iu,*)'len(oldf) = ',neqn
      write(iu,*)'len(of) = ',meqn
      write(iu,*)'len(c) = ',4*meqny+1
      write(iu,*)'len(r) = ',4*meqnx+1
      total = 5*neqn + lrw + 4*meqny + 4*meqnx + meqn + 2
      totalb = 8*total
      write(iu,*)'Total = ',total,'     Bytes = ',totalb
      write(iu,*)'----------------------------------------'

c
      write(iu,*)'file: ',filename2,' will record the solution data'
      write(iu,*)'file: ',filename3,' will be a restart file'
      write(iu,*) 'Sol method for X - Y -Z is ', solverx,solvery,solverz

      return
      end

c----------------------------------------------------------------
