c**********************************************************
c
      Subroutine trid(a,b,c,x,f,n)
c ----------------------------------------------------------
c     solve a tridiagonal system with pivoting
c
c Input:  a,b,c sub, dia, super diagonal
c         f rhs
c         n number of eqs and unknowns
c Output: x solution
c
c ----------------------------------------------------------
c
      integer n,nmax,k,i
      parameter(nmax=1000)
      double precision a(n),b(n),c(n),d(nmax),x(n),f(n)
      double precision amul,temp
c
      if(n.gt.nmax)  stop'n.gt.nmax'
c        
      do i=1,n
         x(i)=f(i) 
         d(i)=0.d0
      enddo
c   
      do k=1,n-1
         if(abs(a(k+1)).gt.abs(b(k))) then 
c     
            temp=b(k)
            b(k)=a(k+1)
            a(k+1)=temp
            temp=c(k)
            c(k)=b(k+1)
            b(k+1)=temp
            temp=x(k)
            x(k)=x(k+1)
            x(k+1)=temp
            if(k.lt.n-1) then 
               temp=d(k)
               d(k)=c(k+1)
               c(k+1)=temp
            endif
         endif	
         if(abs(b(k)).lt.1.e-13) goto 20
         amul=-a(k+1)/b(k)
         b(k+1)=b(k+1)+amul*c(k)
         x(k+1)=x(k+1)+amul*x(k)
         if(k.lt.n-1) c(k+1)=c(k+1)+amul*d(k)
c
      enddo
c
c        back substitution
c
      if(b(n).eq.0.d0) goto 20
      x(n)=x(n)/b(n)
      x(n-1)=(x(n-1)-c(n-1)*x(n))/b(n-1)
      do  k=n-2,1,-1
         x(k)=(x(k)-c(k)*x(k+1)-d(k)*x(k+2))/b(k)
      enddo 
      return
c              the matrix is singular 
 20   print*, 'The matrix is singular'
c 
      return
      end

c********************************************************************
c
      subroutine ntrsol(a,b,c,p,nv,ngrid,miter,ftol,t,dt,
     .     diaga,diagb,diagc,fm,of,u,fp,sol)
c
c--------------------------------------------------------------------
c Tridiagonal solver is a newton solver that solves nonlinear 
c system of equations:
c            F(U) = 0
c whose Jacobian matrix is tridiagonal.
c
c Input: nv, ngrid are the number of variables, grid points
c        miter is the maximum number of iterates for Newton method
c        ftol is the error tolerances for the Newton solver
c        p = (p1,p2,p3,irow,icl)
c        t, dt are the time, the time step size.
c        a, b, c, stand for sub, diag, super, three nb-vectors of blocks, 
c         diagonal blocks. They are setuped for diffusing part of the 
c        Jacobian matrix.
c Output: p in which p1 is the current solutions
c-----------------------------------------------------------------------

      integer k, l, iter, length
      integer nv, ngrid, miter
      integer irow,icol,iplane
      double precision a(nv*ngrid),b(nv*ngrid),c(nv*ngrid)
      double precision diaga(nv*ngrid)
      double precision diagb(nv*ngrid)
      double precision diagc(nv*ngrid)
      double precision p(3*nv*ngrid+2), ftol, t, dt
      double precision fm(nv*ngrid),of(nv*ngrid)
      double precision u(nv), res, sol(nv*ngrid), fp(nv,nv)
c
      length = nv*ngrid              !length of p_i
      icol= idint(p(3*length+1))
      irow= idint(p(3*length+2))
c
      iter = 0                  ! Initialize the counter for the Newton step

 99   call functz(length,3*length+2,p,of)
c
c  Compute the residuals
c
      res = 0.d0
      do l = 1, nv
         do k = 1,ngrid
            if(dabs(of(l+(k-1)*nv)) .gt. res) then
               res=dabs(of(l+(k-1)*nv))
            endif
         enddo
      enddo
c
c If res < ftol, return; otherwise start next iterate
c
      if(res .lt. ftol)go to 101
c
      do k = 1, ngrid
         do l=1,nv
            u(l) = p(l+(k-1)*nv)
         enddo
c
c Add Jacobian of f(u) to the tridiagonal matrix 
c
         iplane = k
         call funcjac(u,irow,icol,iplane,t+dt,fp)

         do l = 1, nv
            diaga(k+(l-1)*ngrid)=-a(k+(l-1)*ngrid)
            diagb(k+(l-1)*ngrid)=-b(k+(l-1)*ngrid) 
     .                               + 0.5*dt*fp(l,l)
            diagc(k+(l-1)*ngrid)=-c(k+(l-1)*ngrid)
            fm(k+(l-1)*ngrid) = -of(l+(k-1)*nv)
         enddo
      enddo

      call trid(diaga,diagb,diagc,sol,fm,length)
c
c     Update the p1 of p
c
      do l = 1, nv
         do k = 1,ngrid
            p(l+(k-1)*nv) = p(l+(k-1)*nv) + sol(k+(l-1)*ngrid)
         enddo
      enddo
c
c     If the number of the iterates does not reach the max iterate 
c     number calculate the residual. Otherwise, return.
c
      iter = iter +1
      if(iter .lt. miter) go to 99
c
 101  miter = iter
c
      return
      end

c***********************************************************************
c
         subroutine rhs_row(irow,iplane,v,r_rhs)
c
c-----------------------------------------------------------------------
c     Compute the right side of equations for the row sweep 
c     (for fixed y(irow),z(iplane)
c
c Inout: irow,iplane 
c        v = (u, u**, u*, oldu)  
c Output: r_rhs = [I+ht*(A1+2*A2+2*A3)/2]*oldu + ht*f(oldu)
c           + bc (on u* for x-direction)
c-----------------------------------------------------------------------
c
         include 'params.inc'
c
c        integer i,ind,l,index, iflg
         integer i,ind,l,index
c        integer irow,icol,iplane
         integer irow,iplane
         double precision v(4*neqn), r_rhs(meqnx) 
c        double precision ht,t,x,y,z,htx,hty,htz
         double precision ht,t,htx,hty,htz
         double precision lambdx, lambdy, lambdz,lambd
         double precision diffx,diffy,diffz
         double precision g1, g2, g3, g4, g5, g6       !functions for BC
         double precision u(nvar),u0(nvar), f_rhs(nvar)
c
         character*10 bc
c
         external g1,g2,g3,g4,g5,g6
c
         common/ttime/t,ht,htx,hty,htz
         common/bound_cond/bc
c
c-----------------------------------------------------------------
c     Compute oldu + ht*f(oldu)
c
         do i = 1, nx
           do l = 1, nvar
              index = i+(l-1)*nx+(irow-1)*meqnx+(iplane-1)*meqnx*ny
              ind = index+3*neqn
              u0(l) = v(ind)
              u(l) = v(index)
c               
              call func(u0,irow,i,iplane,t,f_rhs)  !calculate f(u^n)
           enddo
c
           do l = 1, nvar
              index = i+(l-1)*nx+(irow-1)*meqnx+(iplane-1)*meqnx*ny
              ind = index+3*neqn
              r_rhs(i+(l-1)*nx) = v(ind) + ht*f_rhs(l)
           enddo
        enddo
c
c-----------------------------------------------------------------
c     Compute the elements in the vector for the diffusing 
c     variables, including non-zero boundary condition.
c------------------------------------------------------
c for Neumann BC
c
         if(bc .eq. 'neumann') then 
c
         do l=nstat+1,nvar
            lambdx = htx*par(1+3*(l-1))
            lambdy = hty*par(2+3*(l-1))
            lambdz = htz*par(3+3*(l-1))
            lambd = -lambdx-2.d0*lambdy-2.d0*lambdz
c
            do i = 1,nx
               index = i+(l-1)*nx+(irow-1)*meqnx+(iplane-1)*meqnx*ny
               ind = index+3*neqn
               r_rhs(i+(l-1)*nx) = r_rhs(i+(l-1)*nx) + lambd*v(ind) 
c
c Compute ht*A1/2+lambdx*oldu + bc (on u* for x-direction)
c
               if(i .eq. 1) then                       ! on x = 0
                  diffx = lambdx*(v(ind+1) 
     .                  + hx*(-g1((irow-1)*hy,(iplane-1)*hz,t+ht/3.d0)
     .                       -g1((irow-1)*hy,(iplane-1)*hz,t)))
c
               elseif(i .eq. nx) then                  ! on x = 1
                  diffx = lambdx*(v(ind-1) 
     .                  + hx*(g2((irow-1)*hy,(iplane-1)*hz,t+ht/3.d0)
     .                        +g2((irow-1)*hy,(iplane-1)*hz,t)))
c
               else                                    !inside points
                  diffx = 0.5d0*lambdx*(v(ind-1) + v(ind+1))
               endif
c
c Compute ht*A2+2*lambdy*oldu 
c
               if(irow .eq. 1) then                    ! on y = 0
                  diffy = 2.d0*lambdy*(v(ind+meqnx) 
     .                  - hy*g3((i-1)*hx,(iplane-1)*hz,t))
c
               elseif(irow .eq. ny) then               ! on y = 1
                  diffy = 2.d0*lambdy*(v(ind-meqnx) 
     .                  + hy*g4((i-1)*hx,(iplane-1)*hz,t))
c
               else                                    !inside points
                  diffy = lambdy*(v(ind-meqnx) + v(ind+meqnx))
               endif
c
c Compute ht*A3+2*lambdz*oldu 
c
               if(iplane .eq. 1) then                  ! on z = 0
                  diffz = 2.d0*lambdz*(v(ind+meqnx*ny) 
     .                  - hz*g5((i-1)*hx,(irow-1)*hy,t))
c
               elseif(iplane .eq. nz) then             ! on z = 1
                  diffz = 2.d0*lambdz*(v(ind-meqnx*ny) 
     .                  + hz*g6((i-1)*hx,(irow-1)*hy,t))
c
               else                                    !inside points
                  diffz = lambdz*(v(ind-meqnx*ny) + v(ind+meqnx*ny))
               endif
c
c   Adding diffx,diffy,diffz to r_rhs
c
               r_rhs(i+(l-1)*nx) = r_rhs(i+(l-1)*nx) + 
     .                             diffx + diffy + diffz
            enddo
         enddo
c
c For Dirichlet BC. Assume v is known on boudary.
c
         elseif(bc .eq. 'dirichlet') then
c         
         do l=nstat+1,nvar
            lambdx = htx*par(1+3*(l-1))
            lambdy = hty*par(2+3*(l-1))
            lambdz = htz*par(3+3*(l-1))
            lambd = -lambdx-2.d0*lambdy-2.d0*lambdz
c
            do i = 1,nx
               index = i+(l-1)*nx+(irow-1)*meqnx
     .                 +(iplane-1)*meqnx*ny
               ind = index+3*neqn
               r_rhs(i+(l-1)*nx) = r_rhs(i+(l-1)*nx)+lambd*v(ind) 
c
c Compute ht*A1/2+lambdx*oldu + bc (on u* for x-direction)
c
               if(i .eq. 1) then                         ! on x = hx
                  diffx = 0.5d0*lambdx*(v(ind+1)         !oldu2
     .                 + g1((irow-1)*hy,(iplane-1)*hz,t) !oldu at x=0
     .        + g1((irow-1)*hy,(iplane-1)*hz,t+ht/3.d0)) !u* at x=0
c
               elseif(i .eq. nx) then                    ! on x = 1-hx
                  diffx = 0.5d0*lambdx*(v(ind-1) 
     .                 + g2((irow-1)*hy,(iplane-1)*hz,t) !oldu at x=1
     .        + g2((irow-1)*hy,(iplane-1)*hz,t+ht/3.d0)) !u* at x=1
c
               else
                  diffx = 0.5d0*lambdx*(v(ind-1) + v(ind+1))
               endif
c
c Compute ht*A2+2*lambdy*oldu 
c               
               if(irow .eq. 1) then                      ! on y = hy
                  diffy = lambdy*(v(ind+meqnx)
     .                 + g3((i-1)*hx,(iplane-1)*hz,t))   !oldu at y=0
c
               elseif(irow .eq. ny) then                 ! on y = 1-hy
                  diffy = lambdy*(v(ind-meqnx)
     .                 + g4((i-1)*hx,(iplane-1)*hz,t))   !oldu at y=1
c
               else
                  diffy = lambdy*(v(ind-meqnx) + v(ind+meqnx))
               endif
c
c Compute ht*A3+2*lambdz*oldu 
c
               if(iplane .eq. 1) then                    ! on z = hz
                  diffz = lambdz*(v(ind+meqnx*ny) 
     .                 + g5((i-1)*hx,(irow-1)*hy,t))     !oldu at z=0
c
               elseif(iplane .eq. nz) then               ! on z = 1-hz
                  diffz = lambdz*(v(ind-meqnx*ny) 
     .                 + g6((i-1)*hx,(irow-1)*hy,t))     !oldu at z=0
c
               else
                  diffz = lambdz*(v(ind-meqnx*ny) 
     .                 + v(ind+meqnx*ny))
               endif
c
c   Adding diffx,diffy,diffz to r_rhs
c
               r_rhs(i+(l-1)*nx) = r_rhs(i+(l-1)*nx) 
     .                           + diffx + diffy + diffz
            enddo
         enddo
c
         endif
c
         return
         end

C*********************************************************************
c
         subroutine rhs_col(icol,iplane,v,r_rhs)
c
c--------------------------------------------------------------------
c     Compute the right side of equations for the col sweep
c     (for fixed x(icol),z(iplane)
c
c Input: icol,iplane
c        v = (u, u**, u*, oldu)  
c Output: r_rhs = u* - ht*A2oldu/2
c           + bc (on u** for y-direction)
c-----------------------------------------------------------------
c
         include 'params.inc'

c        integer j,ind0,ind1,l,index, iflg
         integer j,ind0,ind1,l,index
c        integer irow,icol,iplane
         integer icol,iplane
         double precision v(4*neqn), r_rhs(meqny) 
c        double precision ht,t,x,y,z,htx,hty,htz
         double precision ht,t,htx,hty,htz
         double precision lambdy
         double precision diffy
         double precision g1, g2, g3, g4, g5, g6

         character*10 bc

         external g1,g2,g3,g4,g5,g6

         common/ttime/t,ht,htx,hty,htz
         common/bound_cond/bc

c-----------------------------------------------------------
c     Compute the elements in the vector for the non-diffusing 
c     variables.
c
         if(nstat .gt. 0) then
c
         do l = 1, nstat
            do j = 1, ny
               index = icol+(l-1)*nx+(j-1)*meqnx+(iplane-1)*meqnx*ny
               ind1 = index+2*neqn                        !u*
               r_rhs(j+(l-1)*ny) = v(ind1) 
            enddo
         enddo
c
         endif
c
c-----------------------------------------------------------------
c     Compute the elements in the vector for the diffusing 
c     variables, including non-zero boundary condition.
c------------------------------------------------------
c
         do l=nstat+1,nvar
            lambdy = hty*par(2+3*(l-1))
c
c Compute r_rhs at inside points
c
            do j = 2,ny-1
               index = icol+(l-1)*nx+(j-1)*meqnx+(iplane-1)*meqnx*ny
               ind0 = index+3*neqn                        !oldu
               ind1 = index+2*neqn                        !u*
c
c Compute u* + 2*lambdy*oldu
c
               r_rhs(j+(l-1)*ny) = v(ind1) + lambdy*v(ind0) 
c
c Compute -(tau*A2+2*lambdy*oldu)/2 at inside points
c
               diffy = -0.5d0*lambdy*(v(ind0-meqnx) + v(ind0+meqnx))
c
c   Adding diffy to r_rhs
c
               r_rhs(j+(l-1)*ny) = r_rhs(j+(l-1)*ny) + diffy
            enddo
c
c---------------------------------------------------------
c Compute u* - ht*A2oldu/2 on the boundary y = 0
c     + bc (on u** for y-direction)
c----------------------------------------------------------
c   j = 1
            index = icol+(l-1)*nx+(iplane-1)*meqnx*ny
            ind0 = index+3*neqn                        !oldu
            ind1 = index+2*neqn                        !u*
c            
            r_rhs(1+(l-1)*ny) = v(ind1) + lambdy*v(ind0) 
c
c Compute -(ht*A2+2*lambdy*oldu)/2 + bc (on u** for y-direction)
c
c for Neumann BC
c
         if(bc .eq. 'neumann') then 
            diffy = -lambdy*(v(ind0+meqnx) 
     .           - hy*g3((icol-1)*hx,(iplane-1)*hz,t)          !u^n at y=0
     .      + hy*g3((icol-1)*hx,(iplane-1)*hz,t+2.d0*ht/3.d0)) !u** at y=0
c
c for Dirichlet BC
c
         elseif(bc .eq. 'dirichlet') then
            diffy = -0.5d0*lambdy*(v(ind0+meqnx) 
     .      + g3((icol-1)*hx,(iplane-1)*hz,t)                 !u^n at y=0
     .      - g3((icol-1)*hx,(iplane-1)*hz,t+2.d0*ht/3.d0))   !u** at y=0
         endif
c
c   Adding diffy to r_rhs
c
            r_rhs(1+(l-1)*ny) = r_rhs(1+(l-1)*ny) + diffy
c---------------------------------------------------------
c Compute -(ht*A2+2*lambdy*oldu)/2 on the boundary y = 1
c     + bc (on u** for y-direction)
c---------------------------------------------------------
c   j = ny
            index = icol+(l-1)*nx+(ny-1)*meqnx+(iplane-1)*meqnx*ny
            ind0 = index+3*neqn                           !oldu
            ind1 = index+2*neqn                           !u*
c               
            r_rhs(ny+(l-1)*ny) = v(ind1) + lambdy*v(ind0) 
c
c Compute -(ht*A2+2*lambdy*oldu)/2 + bc (on u** for y-direction)
c
c for Neumann BC
c
         if(bc .eq. 'neumann') then 
            diffy = -lambdy*(v(ind0-meqnx) 
     .           + hy*g4((icol-1)*hx,(iplane-1)*hz,t)           !u^n at y=1
     .      - hy*g4((icol-1)*hx,(iplane-1)*hz,t+2.d0*ht/3.d0))  !u** at y=1
c
c for Dirichlet BC
c
         elseif(bc .eq. 'dirichlet') then
            diffy = -lambdy*(v(ind0-meqnx) 
     .           + g4((icol-1)*hx,(iplane-1)*hz,t)           !u^n at y=1
     .      - g4((icol-1)*hx,(iplane-1)*hz,t+2.d0*ht/3.d0))  !u** at y=1
         endif
c
c   Adding diffy to r_rhs

            r_rhs(ny+(l-1)*ny) = r_rhs(ny+(l-1)*ny) + diffy
c
         enddo
c
         return
         end

C*******************************************************************
c
          subroutine rhs_plane(icol,irow,v,r_rhs)
c
c-------------------------------------------------------------------
c     Compute the right side of equations for the z sweep
c     (for fixed x(icol),y(irow)
c
c Input: icol,irow
c        v = (u, u**, u*, oldu)  
c Output: r_rhs = u** - ht*A3oldu/2
c           + bc (on u for y-direction)
c-----------------------------------------------------------------
c
         include 'params.inc'
c
c        integer k,ind0,ind1,l,index, iflg
         integer k,ind0,ind1,l,index
c        integer irow,icol,iplane
         integer irow,icol
         double precision v(4*neqn), r_rhs(meqny) 
c        double precision ht,t,x,y,z,htx,hty,htz
         double precision ht,t,htx,hty,htz
         double precision lambdz
         double precision diffz
         double precision g1, g2, g3, g4, g5, g6
c
         character*10 bc
c
         external g1,g2,g3,g4,g5,g6
c
         common/ttime/t,ht,htx,hty,htz
         common/bound_cond/bc
c
c-----------------------------------------------------------
c     Compute the elements in the vector for the non-diffusing 
c     variables.
c
         if(nstat .gt. 0) then
c
         do l = 1, nstat
            do k = 1, nz
               index = icol+(l-1)*nx+(irow-1)*meqnx+(k-1)*meqnx*ny
               ind1 = index+neqn                        !u**
               r_rhs(k+(l-1)*nz) = v(ind1) 
            enddo
         enddo
c
         endif
c
c-----------------------------------------------------------------
c     Compute the elements in the vector for the diffusing 
c     variables, including non-zero boundary condition.
c------------------------------------------------------
c
         do l=nstat+1,nvar
            lambdz = htz*par(3+3*(l-1))
c
c Compute r_rhs at inside points
c
            do k = 2,nz-1
               index = icol+(l-1)*nx+(irow-1)*meqnx+(k-1)*meqnx*ny
               ind0 = index+3*neqn                        !oldu
               ind1 = index+neqn                          !u**
c
c Compute u** + 2*lambdz*oldu
c
               r_rhs(k+(l-1)*nz) = v(ind1) + lambdz*v(ind0) 
c
c Compute -(ht*A3+2*lambdz*oldu)/2 at inside points
c
               diffz = -0.5d0*lambdz*(v(ind0-meqnx*ny) 
     .               + v(ind0+meqnx*ny))
c
c   Adding diffz to r_rhs
c
               r_rhs(k+(l-1)*nz) = r_rhs(k+(l-1)*nz) + diffz
            enddo

c---------------------------------------------------------
c Compute u** - ht*A3oldu/2 on the boundary z = 0
c     + bc (on u for z-direction)
c----------------------------------------------------------
c   k = 1
            index = icol+(l-1)*nx+(irow-1)*meqnx
            ind0 = index+3*neqn                        !oldu
            ind1 = index+neqn                          !u**
            
            r_rhs(1+(l-1)*nz) = v(ind1) + lambdz*v(ind0) 
c
c Compute -(ht*A3+2*lambdy*oldu)/2 + bc (on u for z-direction)
c
c for Neumann BC
c
         if(bc .eq. 'neumann') then 
            diffz = -lambdz*(v(ind0+meqnx*ny) 
     .      - hz*g5((icol-1)*hx,(irow-1)*hy,t)         !u^n at z=0
     .      + hz*g5((icol-1)*hx,(irow-1)*hy,t+ht))     !u at z=0
c
c for Dirichlet BC
c
         elseif(bc .eq. 'dirichlet') then
            diffz = -0.5d0*lambdz*(v(ind0+meqnx*ny) 
     .      + g5((icol-1)*hx,(irow-1)*hy,t)            !u^n at z=hz
     .      - g5((icol-1)*hx,(irow-1)*hy,t+ht))        !u at z=hz
         endif
c
c   Adding diffz to r_rhs
c
            r_rhs(1+(l-1)*nz) = r_rhs(1+(l-1)*nz) + diffz
c
c---------------------------------------------------------------
c Compute u** - ht*A3oldu/2 on the boundary z = 1
c     + bc (on u for z-direction)
c---------------------------------------------------------------
c   k = nz
            index = icol+(l-1)*nx+(irow-1)*meqnx+(nz-1)*meqnx*ny
            ind0 = index+3*neqn                           !oldu
            ind1 = index+neqn                             !u**
               
            r_rhs(nz+(l-1)*nz) = v(ind1) + lambdz*v(ind0) 

c Compute -(tau*A3+2*lambdz*oldu)/2 + bc (on u for y-direction)
c
c for Neumann BC
c
         if(bc .eq. 'neumann') then 
            diffz = -lambdz*(v(ind0-meqnx*ny) 
     .           + hz*g6((icol-1)*hx,(irow-1)*hy,t)          !u^n at z=1
     .           - hz*g6((icol-1)*hx,(irow-1)*hy,t+ht))      !u at z=1
c
c for Dirichlet BC
c
         elseif(bc .eq. 'dirichlet') then
            diffz = -lambdz*(v(ind0-meqnx*ny) 
     .           + g6((icol-1)*hx,(irow-1)*hy,t)          !u^n at z=1-hz
     .           - g6((icol-1)*hx,(irow-1)*hy,t+ht))      !u at z=1-hz
         endif
c
c   Adding diffy to r_rhs

            r_rhs(nz+(l-1)*nz) = r_rhs(nz+(l-1)*nz) + diffz
c
         enddo
c
         return
         end

c----------------------------------------------------------------
