# /u/sy/beebe/src/benchmarks/fogelson/summary.awk, Thu Dec  2 07:47:21 1993
# Edit by Nelson H. F. Beebe <beebe@plot79.math.utah.edu>
# ========================================================================
# Extract a one-line summary of each benchmark from the output of
# results.awk.
#
# Usage:
#	nawk -f results.awk RESULTS | nawk -f summary.awk
#
# [22-Apr-1997]
# ========================================================================

BEGIN	{ 
		time_cutoff = 35 # Total times below this are ignored (failed jobs?) 
		time_cutoff = 15 # SGI Octane takes as little as 15sec on othmer/1dmidlr
		time_cutoff = 07 # DEC Alpha 500/400 is even faster
	}


/^Benchmark:/		{
				benchmark = substr($0,25)
				sub(/nice time .\//,"",benchmark)
				gsub(/ [<>] *[a-zA-Z0-9.]+/,"",benchmark)
				benchmark = trim(benchmark)
			}

/^Machine:/			{machine = trim(substr($0,25,index($0,"(")-26))}

/^Compilation options:/		{options = trim(substr($0,25))}

/^User \+ System time:/		{
        seconds = trim($5) + 0
	combined = benchmark SUBSEP machine SUBSEP options
	while (combined in time)
	    combined = combined " "
	time[combined] = seconds
	if (seconds > time_cutoff)	# ignore anomalous tiny results
	{
	    if (benchmark in min_time)
	    {
		if (min_time[benchmark] > seconds)
		    min_time[benchmark] = seconds
	    }
	    else
		min_time[benchmark] = seconds
	}
}

END \
{
    HLINE = "====================================================" \
    	    "==================================================="
    print HLINE
    printf("%-15s\t%-23s\t%-39s\t%9s\t%7s\n",    
	"Benchmark", "Machine", "Optimization", "Seconds", "Ratio")
    print HLINE
    SORTPIPE = "sort -t'	' +0 -1 +4nr -5"
    for (combined in time)
    {
	split(combined, triple, SUBSEP)
	# Columns: benchmark machine options time ratio
	if (time[combined] > time_cutoff)
	    printf("%-15s\t%-23s\t%-39s\t%9.2f\t%7.4f\n",
		triple[1], triple[2], substr(trim(triple[3]),1,39), time[combined],
		min_time[triple[1]]/time[combined]) | SORTPIPE
    }
    close(SORTPIPE)
    print HLINE
}

function trim(s)
{
    sub(/^[ \t]+/,"",s)
    sub(/[ \t]$/,"",s)
    return s
}
